import CommandManager
import autil
import Hub
import sys, os
import threading

scriptsPath = 'PyScripts'
sys.path.insert(0, scriptsPath)

class ScriptBase(object):
    def __init__(self):
        pass

    def unload(self):
        pass

    def reload(self):
        pass
    
    def __del__(self):
        pass

loadedScripts = {}
scripts = []
scriptsLock = threading.RLock()

scriptsFile = 'pyScripts.txt'
fileLock = threading.RLock()

def initModule():
    CommandManager.add(CommandManager.Command(key='loadScript',command=loadScript,helpTxt='[scriptname] Loads a Script',category="ScriptManager", helpTxtLong=getScriptsDirList, minLevel=9))
    CommandManager.add(CommandManager.Command(key='unloadScript',command=unloadScript, helpTxt='[scriptname] Unloads a loaded Script',category="ScriptManager", helpTxtLong=getScriptsInfo, minLevel=9))
    CommandManager.add(CommandManager.Command(key='reloadScript',command=reloadScript, helpTxt='[scriptname] Reloads a loaded Script preserving any object data',category="ScriptManager", helpTxtLong=getScriptsInfo, minLevel=9))

    global scripts, loadedScripts
    readScripts = autil.parseJsonFile(os.path.join(Hub.CfgDir , scriptsFile))
    saveAfter = False
    for scriptname in readScripts:
        if scriptname not in loadedScripts:
            scriptsLock.acquire()
            try:
                script = __import__(scriptname)
                reload(script)
                if script.Script:
                    loadedScript = script.Script()
                    loadedScripts[scriptname] = loadedScript
                    scripts.append(scriptname)
                    autil.log(__name__, "The script '" + scriptname + "' has been loaded.")
            except Exception as err:
                autil.log(__name__, "Could not load '" + scriptname + "': " + str(err))
            scriptsLock.release()

def getScriptsDirList():
    dirList = os.listdir(scriptsPath)
    msg = "\nScripts in path(" + scriptsPath + "):\n"
    msg += "Loaded  | Script Name\n"
    for file in dirList:
        if not os.path.isdir(os.path.join(scriptsPath,file)):
            name, ext = os.path.splitext(file)
            if ext == '.py' or ext == '.pyc':
                if (ext == '.pyc' and not os.path.exists(os.path.join(scriptsPath,name+'.py'))) or ext == '.py':
                    if name in scripts:
                        msg += "  [X]     "
                    else:
                        msg += "  [ ]     "
                    msg += name + "\n"
    return msg

def getScriptsInfo():
    msg = "\nCurrently loaded Scripts:\n"
    if scripts:
        for script in scripts:
            msg += script + ".py\n"
    return msg

def loadScript(ent, cmd, params, public):
    if len(params) == 1:
        scriptname = params.pop(0)
        if scriptname not in loadedScripts:
            scriptsLock.acquire()
            loadedScript = None
            try:
                script = __import__(scriptname)
                reload(script)
                if script.Script:
                    loadedScript = script.Script()
                    loadedScripts[scriptname] = loadedScript
                    scripts.append(scriptname)
                    saveScripts()
                    autil.log(__name__, "The script '" + scriptname + "' has been loaded by " + ent.getField("NI"))
                    autil.hub_reply(ent, "The script '" + scriptname + "' has been loaded.")
                else:
                    autil.hub_reply(ent, "The script '" + scriptname + "' doesn't have a Script class. Create class Script(ScriptBase)")
            except Exception as err:
                autil.hub_reply(ent, "Could not load '" + scriptname + "': " + str(err))
            scriptsLock.release()
        else:
            autil.hub_reply(ent, "The script '" + scriptname + "' is already loaded, unload it first")
    else:
        autil.hub_reply(ent, "Invalid paramerts, recieved: " + str(params))
        autil.hub_reply(ent, "Parameters must be [string]")
    return False

def reloadScript(ent, cmd, params, public):
    if len(params) == 1:
        scriptname = params.pop(0)
        if scriptname in loadedScripts:
            try:
                script = __import__(scriptname)
                reload(script)
                loadedScripts[scriptname].__class__ = getattr(script, 'Script')
                loadedScripts[scriptname].reload()
                autil.log(__name__, "The script '" + scriptname + "' has been reloaded by " + ent.getField("NI"))
                autil.hub_reply(ent, "The script '" + scriptname + "' has been reloaded.")
            except Exception as err:
                autil.hub_reply(ent, "Could not reload '" + scriptname + "': " + str(err))
        else:
            autil.hub_reply(ent, "The script '" + scriptname + "' is not loaded. Script changes reloaded automatically upon normal load command")
    else:
        autil.hub_reply(ent, "Invalid paramerts, recieved: " + str(params))
        autil.hub_reply(ent, "Parameters must be [string]")
    return False

def unloadScript(ent, cmd, params, public):
    if len(params) == 1:
        scriptname = params.pop(0)
        if scriptname in loadedScripts:
            scriptsLock.acquire()
            try:
                loadedScripts[scriptname].unload()
                del(loadedScripts[scriptname])
                scripts.remove(scriptname)
                saveScripts()
                autil.log(__name__, "The script '" + scriptname + "' has been unloaded by " + ent.getField("NI"))
                autil.hub_reply(ent, "The script '" + scriptname + "' has been unloaded.")
            except Exception as err:
                autil.hub_reply(ent, "Could not load '" + scriptname + "': " + str(err))
            scriptsLock.release()
        else:
            autil.hub_reply(ent, "The script '" + scriptname + "' doesn't exist in the loaded Scripts")
    else:
        autil.hub_reply(ent, "Invalid paramerts, recieved: " + str(params))
        autil.hub_reply(ent, "Parameters must be [string]")
    return False

def saveScripts():
    fileLock.acquire()
    autil.saveJsonFile(scripts, os.path.join(Hub.CfgDir , scriptsFile))
    fileLock.release()