import pyadchpp as adchpp
import Hub
import autil
import UserManager
import SettingManager
import time

succeeded = None
failed = None

sup = None
inf = None
pas = None

saltPlugData = None

def initModule():
    global succeeded, failed, sup, inf, pas, saltPlugData

    succeeded = (lambda client: True)
    failed = autil.dump

    autil.addSupport("PING")

    sup = autil.handleCommand(onSUP,adchpp.AdcCommand.CMD_SUP)
    inf = autil.handleCommand(onINF,adchpp.AdcCommand.CMD_INF)
    pas = autil.handleCommand(onPAS,adchpp.AdcCommand.CMD_PAS)
        
    saltPlugData = Hub.PluginMgr.registerPluginData()

def onSUP(ent, cmd, ok):
    if not Hub.ClientMgr.verifySUP(ent, cmd):
        return False

    inf = adchpp.AdcCommand(adchpp.AdcCommand.CMD_INF, adchpp.AdcCommand.TYPE_INFO, adchpp.AdcCommand.HUB_SID)

    if ent.getState() != adchpp.Entity.STATE_PROTOCOL or not ent.hasSupport(adchpp.AdcCommand.toFourCC("PING")):
        return True

    #Pinger stuff:
    #if adchpp.cvar.versionString == 'Debug$':
    #print(adchpp.AdcCommand.fromSID(e.getSID()) + " entering IDENTIFY (supports 'PING')")

    hub = Hub.ClientMgr.getEntity(adchpp.AdcCommand.HUB_SID)
        
    ent.send(hub.getSUP())
        
    supCmd = adchpp.AdcCommand(adchpp.AdcCommand.CMD_SID, adchpp.AdcCommand.TYPE_INFO, adchpp.AdcCommand.HUB_SID)
    supCmd.addParam(adchpp.AdcCommand.fromSID(ent.getSID()))
        
    ent.send(supCmd)
    entities = Hub.ClientMgr.getEntities()
        
    uc = len(entities) #user count
    ss = 0 #sharesize
    sf = 0 #files shared
    if uc > 0:
        for entity in entities:
            ss_ = entity.getField("SS")
            if len(ss_) > 0:
                ss += int(ss_)
            sf_ = entity.getField("SF")
            if len(sf_) > 0:
                sf += int(sf_)

        settings = SettingManager.settings

    inf = adchpp.AdcCommand(adchpp.AdcCommand.CMD_INF, adchpp.AdcCommand.TYPE_INFO, adchpp.AdcCommand.HUB_SID)

    hub.getAllFields(inf)

    inf.delParam("DE", 0)
    
    inf.addParam("DE", str(SettingManager.settings['description'].value))
    #add PING-specific information
    inf.addParam("HH" + str(SettingManager.settings['address'].value))
    inf.addParam("WS" + str(SettingManager.settings['website'].value))
    inf.addParam("NE" + str(SettingManager.settings['network'].value))
    inf.addParam("OW" + str(SettingManager.settings['owner'].value))
    inf.addParam("UC" + str(uc))
    inf.addParam("SS" + str(ss))
    inf.addParam("SF" + str(sf))
    inf.addParam("UP" + str(Hub.core.getUpTime()))
    if 'maxusers' in SettingManager.settings and int(SettingManager.settings['maxusers'].value) > 0:
            inf.addParam("MC" + str(SettingManager.settings['maxusers'].value))

    ent.send(inf)
    ent.setState(adchpp.Entity.STATE_IDENTIFY)

    return False

def verify_info(c, cid, nick):
    if not cid or len(cid) == 0:
        if not c.getCID().isZero():
            cid = c.getCID().toBase32()
        
    if not nick or len(nick) == 0:
        nick = c.getField("NI")
            
    if len(nick) == 0 or len(cid) == 0:
        autil.dump(c, adchpp.AdcCommand.ERROR_PROTOCOL_GENERIC, adchpp.Util.REASON_NICK_INVALID, "No valid nick/CID supplied")
        return False

    if SettingManager.settings['maxnicklength'].value > 0 and len(nick) > SettingManager.settings['maxnicklength'].value:
        autil.dump(c, adchpp.AdcCommand.ERROR_PROTOCOL_GENERIC, adchpp.Util.REASON_NICK_INVALID, "Your nick (" + nick + ") is too long, it must contain " + str(SettingManager.settings['maxnicklength'].value) + " characters max")
        return False
            
    return True

def check_max_users():
    if SettingManager.settings['maxusers'].value == -1:
        return None, None

    if SettingManager.settings['maxusers'].value == 0:
            return adchpp.AdcCommand.ERROR_REGGED_ONLY, "Only registered users are allowed in here"

    count = len(Hub.ClientMgr.getEntities())
    if count >= SettingManager.settings['maxusers'].value:
        return adchpp.AdcCommand.ERROR_HUB_FULL, "Hub full, please try again later"
        
    return None, None
        
def onINF(ent, cmd, ok):
    if not ok:
        return ok
        
    c = ent.asClient()
        
    if not c or c.getState() != adchpp.Entity.STATE_IDENTIFY:
        return True
        
    foundn, nick = cmd.getParam("NI", 0)
    foundc, cid = cmd.getParam("ID", 0)
    if not verify_info(c, cid, nick):
        return False
    
    if not foundn or not foundc:
        return failed(c, adchpp.AdcCommand.ERROR_PROTOCOL_GENERIC, adchpp.Util.REASON_NICK_INVALID , "No valid nick/CID supplied")
        
    user = UserManager.findUser(nick, cid)
    if not user or user.level == 0:
        #non-reg user
        code, err = check_max_users()
        if code:
            autil.dump(c, code, adchpp.Util.REASON_HUB_FULL, err)
            return False
        
    if not user:
        return True

    c.setFlag(adchpp.Entity.FLAG_REGISTERED)
        
    if not Hub.ClientMgr.verifyINF(c, cmd):
        return False

    c.setPluginData(saltPlugData, (Hub.ClientMgr.enterVerify(c, True), str(user.password)) )

    #salt = str(user.salt)
    #passhash = str(user.passhash)

    #Hub.ClientMgr.setState(c, adchpp.Entity.STATE_VERIFY);
    #c.setPluginData(saltPlugData, (salt, passhash)) #even though don't need salt in onPAS, seems like I need to send a tuple or everything gets really messed up

    #cmd = adchpp.AdcCommand(adchpp.AdcCommand.CMD_GPA, adchpp.AdcCommand.TYPE_INFO, adchpp.AdcCommand.HUB_SID)
    #cmd.addParam(salt)
    #c.send(cmd)

    return False
    
def onPAS(ent, cmd, ok):
    if not ok:
        return ok
        
    c = ent.asClient()
 
    if c.getState() != adchpp.Entity.STATE_VERIFY:
        return failed(c, adchpp.AdcCommand.ERROR_PROTOCOL_GENERIC, adchpp.Util.REASON_BAD_STATE, "Not in VERIFY state")
        
    salt, password = c.getPluginData(saltPlugData)
    #salt, passhash = c.getPluginData(saltPlugData)
        
    if not salt:
        return failed(c, adchpp.AdcCommand.ERROR_PROTOCOL_GENERIC, adchpp.Util.REASON_PLUGIN, "You didn't get any salt?")
        
    c.setPluginData(saltPlugData, None)
        
    cid = c.getCID()
    nick = c.getField("NI")

    if not Hub.ClientMgr.verifyPassword(c, password, salt, cmd.getParam(0)):
    #if cmd.getParam(0) != passhash:
        autil.log(__name__ + " onPas", 'Invalid password for user "' + nick + '"');
        return failed(c, adchpp.AdcCommand.ERROR_BAD_PASSWORD, adchpp.Util.REASON_PLUGIN, "Invalid password")

    if not Hub.ClientMgr.verifyOverflow(c):
        return False

    user = UserManager.findUser(nick, cid.toBase32()) 

    updateOk, message = UserManager.updateUser(user, cid.toBase32(), nick)

    if not updateOk:
        failed(c, adchpp.AdcCommand.ERROR_PROTOCOL_GENERIC, adchpp.Util.REASON_PLUGIN, message)
        return False

    if message:
        autil.hub_reply(c, message)

    UserManager.setEntFlags(ent)
    user.lasttime = time.time()
    UserManager.userChangesMade()

    Hub.ClientMgr.enterNormal(c, True, True)
        
    succeeded(c)
    return False