import pyadchpp as adchpp
import autil
import re

BASE32_CHARS = "[2-7a-zA-Z]"
any = re.compile(".*")
nonempty = re.compile(".+")
sta_code = re.compile("[0,1,2][0-9]{2}")
sid = re.compile(BASE32_CHARS + "{4}")
tth = re.compile(BASE32_CHARS + "{39}")
integer = re.compile("[\\-0-9]+")
base32 = re.compile(BASE32_CHARS + "+");
boolean = re.compile("[1]?")
    
fields = {
    adchpp.AdcCommand.CMD_INF : {
        "ID" : tth,
        "PD": tth,
        "I4": re.compile("(([0-1]?[0-9]{1,2}[.])|(2[0-4][0-9][.])|(25[0-5][.])){3}(([0-1]?[0-9]{1,2})|(2[0-4][0-9])|(25[0-5]))"),
        "I6": re.compile("[0-9a-fA-F:]+"), # This could be better
        "U4": integer,
        "U6": integer,
        "SS": integer,
        "SF": integer,
        "US": integer,
        "DS": integer,
        "SL": integer,
        "AS": integer,
        "AM": integer,
        "NI": nonempty,
        "HN": integer,
        "HR": integer,
        "HO": integer,
        "OP": boolean,
        "AW": re.compile("1|2"),
        "BO": boolean,
        "HI": boolean,
        "HU": boolean,
        "SU": re.compile("[0-9A-Z,]+"),
    },
    adchpp.AdcCommand.CMD_MSG : {
        "PM": sid,
        "ME": boolean,
    },
         
    adchpp.AdcCommand.CMD_SCH : {
        "AN": nonempty,
        "NO": nonempty,
        "EX": nonempty,
        "LE": integer,
        "GE": integer,
        "EQ": integer,
        "TO": nonempty,
        "TY": re.compile("1|2"),
        "TR": tth,
    },
        
    adchpp.AdcCommand.CMD_RES : {
        "FN": nonempty,
        "SI": integer,
        "SL": integer,
        "TO": nonempty,
        "TR": tth,
        "TD": integer,
    }
}
    
params = {
    adchpp.AdcCommand.CMD_STA: (sta_code, any),
    adchpp.AdcCommand.CMD_MSG: (any,),
    adchpp.AdcCommand.CMD_CTM: (any, integer, any),
    adchpp.AdcCommand.CMD_RCM: (any, any),
    adchpp.AdcCommand.CMD_PAS: (base32,)
}

succeeded = None
failed = None
inf = None
 
def initModule():
    global succeeded, failed, inf

    succeeded = (lambda client: True)
    failed = autil.dump
    inf = autil.handleCommand(validate, adchpp.AdcCommand.CMD_INF)
        
def validate(ent, cmd, ok):
    c = ent.asClient()
    #print(cmd.toString())
    if ok and cmd.getCommand() in params:
        ok &= validateParam(c, cmd, params[cmd.getCommand()])
        
    if ok and cmd.getCommand() in fields:
        ok &= validateFields(c, cmd, fields[cmd.getCommand()])
        
    return ok & succeeded(c)

def validateParam(c, cmd, params):
    if len(cmd.getParameters()) < len(params):
        autil.log("Access-validateParam", "Too few parameters for " + str(cmd.getCommand()))
        autil.log("Access-validateParam", cmd.toString())
        return failed(c, adchpp.AdcCommand.ERROR_PROTOCOL_GENERIC, adchpp.Util.REASON_INVALID_COMMAND_TYPE, "Too few parameters for " + str(cmd.getCommand()))
        
    #print(params)
    for i, param in enumerate(params):
        if not param.match(cmd.getParam(i)):
            autil.log("Access-validateParam", cmd.getParam(i) + " doesn't match " + param)
            autil.log("Access-validateParam", cmd.toString())
            return failed(c, adchpp.AdcCommand.ERROR_PROTOCOL_GENERIC, adchpp.Util.REASON_INVALID_COMMAND_TYPE, cmd.getParam(i) + " doesn't match " + param)
    return True
    
def validateFields(c, cmd, fields):
    for field in cmd.getParameters():
        if field[0:2] in fields:
            r = fields[field[0:2]]
            if field[2:] and not r.match(field[2:]):
                autil.log("Access-validateFields", field + " doesn't match " + str(r.pattern) + ", value: " + str(field[0:2]))
                autil.log("Access-validateFields", cmd.toString())
                return failed(c, adchpp.AdcCommand.ERROR_PROTOCOL_GENERIC, adchpp.Util.REASON_INVALID_COMMAND_TYPE, field + " doesn't match " + str(r.pattern) + ", value: " + str(field[0:2]))
    return True