import pyadchpp as adchpp
import UserManager
import SettingManager
import autil

inf = None

def initModule():
    global inf

    inf = autil.handleCommand(onINF,adchpp.AdcCommand.CMD_INF)

    SettingManager.add(SettingManager.Setting(key='maxhubscount',alias=['maxhubs'],helpTxt='maximum number of connected hubs allowed, 0 = disabled',value=0) )
    SettingManager.add(SettingManager.Setting(key='maxhubslotratio',alias=['maxhsr'],helpTxt='maximum hub/slot ratio allowed, 0 = disabled',value=0) )
    SettingManager.add(SettingManager.Setting(key='maxsharesize',alias=['maxss'],helpTxt='maximum share size allowed in bytes, 0 = disabled',value=0) )
    SettingManager.add(SettingManager.Setting(key='maxslots',alias=['maxsl'],helpTxt='maximum number of opened upload slots allowed, 0 = disabled',value=0) )
    SettingManager.add(SettingManager.Setting(key='minhubslotratio',alias=['minhsr'],helpTxt='minimum hub/slot ratio required, 0 = disabled',value=0) )
    SettingManager.add(SettingManager.Setting(key='minsharesize',alias=['minss'],helpTxt='minimum share size allowed in bytes, 0 = disabled',value=0) )
    SettingManager.add(SettingManager.Setting(key='minslots',alias=['minsl'],helpTxt='minimum number of opened upload slots required, 0 = disabled',value=0) )
    SettingManager.add(SettingManager.Setting(key='rulesredirect',helpTxt="redirect address when rules like slots/hubs/sharesize aren't obeyed, empty = users will be disconnected only",value="") )

def limit_dump(ent, msg):
    address = SettingManager.settings['rulesredirect'].value
    if len(address) > 0:
        dropmsg = "You are redirected because: " + msg
        autil.dump(ent, adchpp.AdcCommand.ERROR_BANNED_GENERIC, adchpp.Util.REASON_PLUGIN, (lambda cmd: cmd.addParam("MS", dropmsg).addParam("RD" + address)))
    else:
        dropmsg = "You are disconnected because: " + msg
        autil.dump(ent, adchpp.AdcCommand.ERROR_BANNED_GENERIC, adchpp.Util.REASON_PLUGIN, (lambda cmd: cmd.addParam("MS", dropmsg).addParam("TL" + str(-1))))

def onINF(ent, cmd, ok):
    if UserManager.getLevel(ent) >= SettingManager.settings['level_op'].value:
        return True

    def paramToNum(param):
        if param[0]:
            return autil.toNumber(param[1])
        return 0

    c = ent.asClient()

    ss = paramToNum(cmd.getParam("SS", 0)) or autil.toNumber(c.getField("SS"))
                
    if SettingManager.settings['minsharesize'].value > 0 and ss < SettingManager.settings['minsharesize'].value:
        msg = "Your share size (" + adchpp.Util.formatBytes(ss) + ") is too low, the minimum required size is " + adchpp.Util.formatBytes(SettingManager.settings['minsharesize'].value)
        limit_dump(c, msg)
        return False

    if SettingManager.settings['maxsharesize'].value > 0 and ss > SettingManager.settings['minsharesize'].value:
        msg = "Your share size (" + adchpp.Util.formatBytes(ss) + ") is too high, the maximum allowed size is " + adchpp.Util.formatBytes(SettingManager.settings['minsharesize'].value)
        limit_dump(c, msg)
        return False

    sl = paramToNum(cmd.getParam("SL", 0)) or autil.toNumber(c.getField("SL"))
        
    if SettingManager.settings['minslots'].value > 0 and sl < SettingManager.settings['minslots'].value:
        msg = "Your have too few upload slots open (" + str(sl) + "), the minimum required is " + str(SettingManager.settings['minslots'].value)
        limit_dump(c, msg)
        return False

    if SettingManager.settings['maxslots'].value > 0 and sl > SettingManager.settings['maxslots'].value:
        msg = "Your have too many upload slots open (" + str(sl) + "), the maximum allowed is " + str(SettingManager.settings['maxslots'].value)
        return False

    h1 = paramToNum(cmd.getParam("HN", 0)) or autil.toNumber(c.getField("HN"))
    h2 = paramToNum(cmd.getParam("HR", 0)) or autil.toNumber(c.getField("HR"))
    h3 = paramToNum(cmd.getParam("HO", 0)) or autil.toNumber(c.getField("HO"))
    h = 0
    if h1 and h2 and h3:
        h = h1 + h2 + h3
        if SettingManager.settings['maxhubscount'].value > 0 and h > SettingManager.settings['maxhubscount'].value:
            msg = "The number of hubs you're connected to (" + str(h) + ") is too high, the maximum allowed is " + str(SettingManager.settings['maxhubscount'].value)
            limit_dump(c, msg)
            return False

    if sl and h and sl > 0 and h > 0: # The count may not be correct on the first INF
        r = sl / h
        if SettingManager.settings['minhubslotratio'].value > 0 and r < SettingManager.settings['minhubslotratio'].value:
            msg = "Your hubs/slots ratio (" + str(r) + ") is too low, you must open up more upload slots or disconnect from some hubs to achieve a ratio of " + str(SettingManager.settings['minhubslotratio'].value)
            limit_dump(c, msg)
            return False

        if SettingManager.settings['maxhubslotratio'].value > 0 and r > SettingManager.settings['minhubslotratio'].value:
            msg = "Your hubs/slots ratio (" + str(r) + ") is too high, you must reduce your open upload slots or connect to more hubs to achieve a ratio of " + str(SettingManager.settings['maxhubslotratio'].value)
            limit_dump(c, msg)
            return False

    return True