import CommandManager
import SettingManager
import pyadchpp as adchpp
import Hub
import autil
from ScriptManager import ScriptBase
import sys
import urllib

sys.path.insert(0, 'PyScripts/data/wolframalpha')

import wap

class Script(ScriptBase):

    server = 'http://api.wolframalpha.com/v1/query.jsp'

    def __init__(self):
        self.waCmd = CommandManager.Command(key='wolframalpha', alias=['walpha','wolfram', 'wa'],category="ScriptCommands",command=self.doQuery, helpTxt="string - Queries Wolfram Alpha")
        CommandManager.add(self.waCmd)

        self.waAppID = SettingManager.Setting(key='wolfram_appid', helpTxt="You need to sign up for and get an APP ID on http://products.wolframalpha.com/api/webserviceapi.html to query Wolfram Alpha", value="")
        self.waPublicLength = SettingManager.Setting(key='wolfram_maxlines', helpTxt="Max size (in lines) of the message users can send to public without it being truncated", value=30)
        SettingManager.add(self.waAppID)
        SettingManager.add(self.waPublicLength)

    def unload(self):
        self.waCmd.release()
        self.waAppID.release()
        self.waPublicLength.release()

    def reload(self):
        self.waCmd.command = self.doQuery

    def doQuery(self, ent, cmd, params, public):
        if not params:
            autil.hub_reply(ent, "You need to supply a query")

        waeo = wap.WolframAlphaEngine(self.waAppID.value, self.server)
        query = waeo.CreateQuery(urllib.quote(params[0]))
        result = waeo.PerformQuery(query)
        waeqr = wap.WolframAlphaQueryResult(result)

        if waeqr.IsSuccess():
            returnResult = "http://www.wolframalpha.com/input/?i=" + urllib.quote(params[0]) + "  (Query time: " + str(autil.toUTF8(waeqr.Timing()[0])) + "s)\n"
            pods = waeqr.Pods()

            for pod in pods:
                waep = wap.Pod(pod)

                if waep.IsError()[0] == "false":
                    if waep.Title()[0]:
                        returnResult += "\r[" + str(autil.toUTF8(waep.Title()[0])) + "]\n"
                    
                    subpods = waep.Subpods()
                    for subpod in subpods:
                        waesp = wap.Subpod(subpod)

                        if waesp.Title()[0]:
                            returnResult += "{" + str(autil.toUTF8(waesp.Title()[0])) + "}  "
                        if waesp.Plaintext()[0]:
                            returnResult += str(autil.toUTF8(waesp.Plaintext()[0])) + "\n"
                        else:
                            returnResult += "<<See link for details>>\n"

                if public and (returnResult.count('\n') + returnResult.count('\r')) >= self.waPublicLength.value:
                    returnResult += "<<TRUNCATED...>>"
                    break

            if public:
                autil.ent_reply(ent, "[Wolfram|Alpha] " + returnResult);
            else:
                autil.hub_reply(ent, returnResult);
        else:
            autil.hub_reply(ent, "There was an error with your query: [" + str(waeqr.ErrorCode()) + "] " + str(waeqr.Error()))